{ OOP-Deklaration des Graphen                                    <U_graph.Pas> }
{
Dokumentation der Methode der Klasse TGraphNode
-----------------------------------------------
Konstruktor create (pName: string)
nachher	        Ein Knoten mit dem Namen pName wurde erzeugt. Es gibt keine
                Kanten, die diesen Knoten mit anderen Knoten verbinden.
                Der Knoten ist nicht markiert.

Auftrag insertEdge (pEdge: TEdge)
nachher 	Der Knoten hat die Kante pEdge zu einem Nachbarknoten.

Auftrag mark
nachher 	Der Knoten ist markiert.

Auftrag unmark
nachher 	Der Knoten ist nicht markiert.

Anfrage isMarked: boolean
nachher 	Die Anfrage liefert den Wert true, wenn der Knoten markiert ist,
                sonst liefert sie den Wert false.

Anfrage edges: TList
nachher 	Die Anfrage liefert die Liste der Kanten des Knotens.
                Die Inhalte der Listenknoten sind Objekte der Klasse TEdge.
                Hat der Knoten keine Kanten, wird nil zurckgeliefert.

Anfrage name: string
nachher 	Die Anfrage liefert den Namen des Knotens.

Destruktor destroy
nachher 	Der Graphknoten existiert nicht mehr.



Dokumentation der Methoden der Klasse TEdge
--------------------------------------------
Konstruktor create (pNeighbour: TGraphNode; pWeight: double)
nachher 	Eine Kante mit dem Nachbarknoten pNeighbour und dem Gewicht
                pWeight wurde erzeugt.

Anfrage neighbour: TGraphNode
nachher 	Die Anfrage liefert den Nachbarknoten des Knotens,
                zu dem die Kante gehrt.

Anfrage weight: double
nachher 	Die Anfrage liefert das Gewicht der Kante.

Destruktor destroy
nachher 	Die Kante existiert nicht mehr.


Dokumentation der Methoden der Klasse TGraph
--------------------------------------------
Konstruktor create
nachher 	Ein neuer Graph wurde erzeugt. Er enthlt keine Knoten.

Auftrag insertNode (pName: string)
vorher 	        Der Graph hat keinen Knoten mit dem Namen pName.
nachher	        Im Graph existiert ein Knoten mit dem Namen pName.

Auftrag connectNodes (pName1, pName2: string; pWeight: double)
vorher 	        Der Graph enthlt Knoten mit den Namen pName1 und pName2 und
                es gibt keine Kante zwischen diesen beiden Knoten.
nachher 	Die Knoten mit Namen pName1 und pName2 sind durch eine Kante
                verbunden, die das Gewicht pWeight hat. pName1 ist also
                Nachbarknoten von pName2 und umgekehrt.

Anfrage searchNode (pName: string): TGraphNode
nachher 	Die Anfrage liefert den Knoten mit dem Namen pName, wenn er
                im Graphen gefunden wurde, andernfalls wird nil zurckgegeben.

Anfrage allMarked: boolean
nachher 	Die Anfrage liefert den Wert true, wenn alle Knoten des Graphen
                markiert sind, sonst liefert sie den Wert false.

Anfrage hasEdge (pNode1, pNode2: TGraphNode): boolean
nachher 	Die Anfrage liefert den Wert true, wenn die beiden Knoten
                pNode1 und pNode2 im Graphen enthalten sind und sie durch eine
                Kante verbunden sind, sonst liefert sie den Wert false.

Anfrage nodes: TList
nachher	        Die Anfrage liefert eine Liste der Knoten des Graphen.

Destruktor destroy
nacher	        Der Graph existiert nicht mehr.


{ ----------------------------------------  Version 1.0 (Garmann) 16.10.2006 - }

unit U_graph;

interface

uses U_list;

type
     TEdge = class;

     TGraphNode = class
     private
        pName: string;
        pEdges: TLinList;
        pIsMarked: boolean;
     public
        constructor create (pName: string);
        procedure insertEdge (pEdge: TEdge);
        procedure mark;
        procedure unmark;
        function isMarked: boolean;
        function edges: TLinList;
        function name: string;
        destructor destroy; override;
     end;

     TEdge = class
     private
        pWeight: double;
        pNeighbour: TGraphNode;
     public
        constructor create (pNeighbour: TGraphNode; pWeight: double);
        function neighbour: TGraphNode;
        function weight: double;
     end;

     TGraph = class
     private
        pNodes: TLinList;
     public
        constructor create;
        procedure insertNode (pName: string);
        procedure connectNodes (pName1, pName2: string; pWeight: double);
        function searchNode (pName: string): TGraphNode;
        function allMarked: boolean;
        function hasEdge (pNode1, pNode2: TGraphNode): boolean;
        function nodes: TLinList;
        destructor destroy; override;
     end;

implementation

constructor TGraphNode.create (pName: string);
begin
  self.pName:= pName;
  self.pEdges:= TLinList.Create;
  self.pIsMarked:=false;
end;

procedure TGraphNode.insertEdge (pEdge: TEdge);
begin
  pEdges.insertBehind(pEdge);
end;

procedure TGraphNode.mark;
begin
  pIsMarked:= true;
end;

procedure TGraphNode.unmark;
begin
  pIsMarked:= false;
end;

function TGraphNode.isMarked: boolean;
begin
  result:= pIsMarked;
end;

function TGraphNode.edges: TLinList;
begin
  result:= pEdges;
end;

function TGraphNode.name: string;
begin
  result:= pName;
end;

destructor TGraphNode.destroy;
begin
  pEdges.Free;
end;


constructor TEdge.create (pNeighbour: TGraphNode; pWeight: double);
begin
  self.pWeight:= pWeight;
  self.pNeighbour:=pNeighbour;
end;

function TEdge.neighbour: TGraphNode;
begin
  result:= pNeighbour;
end;

function TEdge.weight: double;
begin
  result:= pWeight;
end;



constructor TGraph.create;
begin
  pNodes:= TLinList.create;
end;

procedure TGraph.insertNode (pName: string);
var Node: TGraphNode;
begin
  Node:= TGraphNode.create(pName);
  pNodes.insertBehind(Node);
end;

procedure TGraph.connectNodes (pName1, pName2: string; pWeight: double);
var Node1, Node2: TGraphNode;
    Edge1, Edge2: TEdge;
begin
  Node1:= searchNode(pName1);
  Node2:= searchNode(pName2);
  Edge1:= TEdge.create(Node2,pWeight);
  Node1.insertEdge(Edge1);
  Edge2:= TEdge.create(Node1,pWeight);
  Node2.insertEdge(Edge2);
end;

function TGraph.searchNode (pName: string): TGraphNode;
begin
  result:= nil;
  pNodes.toFirst;
  while not Nodes.isBehind
  do begin
       if TGraphNode(pNodes.getItem).name=pName
       then result:= TGraphNode(pNodes.getItem);
       pNodes.next;
     end;
end;

function TGraph.allMarked: boolean;
begin
  result:= true;
  pNodes.toFirst;
  while not pNodes.isBehind
  do begin
       result:= result and TGraphNode(pNodes.getItem).isMarked;
       pNodes.next;
     end;
end;

function TGraph.hasEdge (pNode1, pNode2: TGraphNode): boolean;
begin
  result:= false;
  pNode1.edges.toFirst;
  while not pNode1.edges.isBehind
  do begin
       result:= result or ( TEdge(pNode1.edges.getItem).neighbour = pNode2 );
       pNode1.edges.next;
     end;
end;

function TGraph.nodes: TLinList;
begin
  result:= pNodes;
end;

destructor TGraph.destroy;
begin
  pNodes.Free;
end;


end.
