{ OOP-Deklaration der linearen Liste                              <U_list.Pas> }
{                                                                              }
{ Dokumentation der Methoden der Klasse TList                                  }
{                                                                              }
{ Konstruktor create                                                           }
{ Nachher: Eine leere Liste ist angelegt. Der interne Positionszeiger steht    }
{          vor der leeren Liste                                                }
{                                                                              }
{ Anfrage isEmpty: boolean                                                     }
{ Nachher: Die Anfrage liefert den Wert true, wenn die Liste keine Elemente    }
{          enthlt, sonst liefert sie den Wert false.                          }
{                                                                              }
{ Anfrage isBefore : Boolean                                                   }
{ Nachher: Die Anfrage liefert den Wert true, wenn der Positionszeiger         }
{          vor dem ersten Listenelement oder vor der leeren Liste steht,       }
{          sonst liefert sie den Wert false.                                   }
{                                                                              }
{ Anfrage isBehind: boolean                                                    }
{ Nachher: Die Anfrage liefert den Wert true, wenn der Positionszeiger         }
{          hinter dem letzten Listenelement oder hinter der leeren Liste steht,}
{          sonst liefert sie den Wert false.                                   }
{                                                                              }
{ Auftrag next                                                                 }
{ Nachher: Der Positionszeiger ist um eine Position in Richtung Listenende     }
{          weitergerckt, d.h. wenn er vor der Liste stand, wird das Element   }
{          am Listenanfang zum aktuellen Element, ansonsten das jeweils        }
{          nachfolgende Listenelement.                                         }
{          Stand der Positionszeiger auf dem letzten Listenelement, befindet   }
{          er sich jetzt hinter der Liste. Befand er sich hinter der Liste,    }
{          hat er sich nicht verndert.                                        }
{                                                                              }
{ Auftrag previous                                                             }
{ Nachher: Der Positionszeiger ist um eine Position in Richtung Listenanfang   }
{          weitergerckt, d.h. wenn er hinter der Liste stand, wird das Element}
{          am Listenende zum aktuellen Element, ansonsten das jeweils          }
{          vorhergehende Listenelement.                                        }
{          Stand der Positionszeiger auf dem ersten Listenelement, befindet er }
{          sich jetzt vor der Liste. Befand er sich vor der Liste,             }
{          hat er sich nicht verndert.                                        }
{                                                                              }
{ Auftrag toFirst                                                              }
{ Nachher: Der Positionszeiger steht auf dem ersten Listenelement. Falls die   }
{          Liste leer ist, befindet er sich jetzt hinter der Liste.            }
{                                                                              }
{ Auftrag toLast                                                               }
{ Nachher: Der Positionszeiger steht auf dem letzten Listenelement. Falls      }
{          die Liste leer ist, befindet er sich jetzt vor der Liste.           }
{                                                                              }
{ Anfrage getItem: TObject                                                     }
{ Nachher: Die Anfrage liefert den Wert des aktuellen Listenelements bzw.      }
{          nil, wenn die Liste keine Elemente enthlt oder der Positionszeiger }
{          vor oder hinter der Liste steht.                                    }
{                                                                              }
{ Auftrag update (pObject: TObject)                                            }
{ Vorher:  Die Liste ist nicht leer. Der Positionszeiger steht nicht vor oder  }
{          hinter der Liste.                                                   }
{ Nachher: Der Wert des Listenelements an der aktuellen Position ist durch     }
{          pObject ersetzt.                                                    }
{                                                                              }
{ Auftrag insertBefore (pObject:TObject)                                       }
{ Vorher:  Der Positionszeiger steht nicht vor der Liste.                      }
{ Nachher: Ein neues Listenelement mit dem entsprechenden Objekt ist angelegt  }
{          und vor der aktuellen Position in die Liste eingefgt worden.       }
{          Der Positionszeiger steht hinter dem eingefgten Element.           }
{                                                                              }
{ Auftrag insertBehind (pObject: TObject)                                      }
{ Vorher:  Der Positionszeiger steht nicht hinter der Liste.                   }
{ Nachher: Ein neues Listenelement mit dem entsprechenden Objekt ist angelegt  }
{          und hinter der aktuellen Position in die Liste eingefgt worden.    }
{          Der Positionszeiger steht vor dem eingefgten Element.              }
{                                                                              }
{ Auftrag delete                                                               }
{ Vorher:  Der Positionszeiger steht nicht vor oder hinter der Liste.          }
{ Nachher: Das aktuelle Listenelement ist gelscht. Der Positionszeiger steht  }
{          auf dem Element hinter dem gelschten Element, bzw. hinter der      }
{          Liste, wenn das gelschte Element das letzte Listenelement war.     }
{          Das Inhaltsobjekt des gelschten Listenelements existiert weiterhin.}
{                                                                              }
{ Destruktor destroy                                                           }
{ Nachher: Die Liste existiert nicht mehr                                      }
{                                                                              }
{ ----------------------------------------  Version 1.0 (Garmann) 09.08.2006 - }

unit U_list;

interface

type
     TElement = class
       private
       pNext: TElement;
       pInhalt: TObject;
       public
       constructor create; overload;
       constructor create(Inhalt: TObject); overload;
       function Inhalt: TObject;
       procedure setInhalt(Inhalt: TObject);
       function Next: TElement;
       procedure setNext(Element: TElement);
     end;

     TLinList = class                           { Liste ber Urtyp TElement    }
       private
         pAnfang, pAktuell: TElement;           { "Zeiger" auf erstes/         }
                                                { aktuelles Element            }
         pIsBefore: boolean;                    { befindet sich Akt. vor Liste }
         pIsBehind: boolean;                    { befindet sich Akt. hinter L. }
       public
         constructor create;                    { setzt Anfang, Aktuell auf NIL}
         destructor destroy; override;          { lscht alle vorh. Elemente   }
       { Navigationsroutinen }
         procedure next;                        { Aktuell einen weiter bewegen }
         procedure previous;                    { Aktuell einen zurck bewegen }
         procedure toFirst;                     { Aktuell an den Anfang        }
         procedure toLast;                      { Aktuell an das Ende          }
       { Datensatzoperationen }
         procedure insertBefore(pObject: TObject); { Datensatz vor Aktuell einf.  }
         procedure insertBehind(pObject: TObject); { Datensatz nach Aktuell einf. }
         procedure delete;                      { Aktuellen Datensatz loeschen }
         procedure update(pObject: TObject);    { Ersetzt den aktuellen Inhalt }
       { Informationsroutinen }
         function isBehind: boolean;            { Aktuell hinter dem Ende?     }
         function isBefore: boolean;            { Aktuell vor dem Anfang=      }
         function isEmpty: boolean;             { Anzahl = 0?                  }
         function getItem: TObject;             { Aktuelles Element ausgeben   }
     end;

implementation

{ **** Routinen fr TElement **** }

constructor TElement.create;                { Konstruktor                      }
begin
  inherited create;                         { ererbten Konstruktor aufrufen    }
  pNext:= NIL;                              { und Nachfolger initialisieren    }
end;

constructor TElement.create(Inhalt: TObject);{ Konstruktor mit Inhalt          }
begin
  inherited create;                         { ererbten Konstruktor aufrufen    }
  pNext:= NIL;                              { und Nachfolger initialisieren    }
  pInhalt:= Inhalt;
end;

function TElement.Inhalt: TObject;          { Rckgabe des Inhalts eines Elem. }
begin
  result:= pInhalt;
end;

procedure TElement.setInhalt(Inhalt: TObject); { Setzen des Inhaltsobjekts     }
begin
  pInhalt:= Inhalt;
end;

function TElement.Next: TElement;           { Rckgabe des Nachfolgerelements  }
begin
  result:= pNext;
end;

procedure TElement.setNext(Element: TElement); { Setzen des Nachfolgerelements }
begin
  pNext:= Element;
end;



{ **** Routinen fr TLinList **** }

constructor TLinList.create;                { Konstruktor                      }
begin
  inherited create;                         { ererbten Konstruktor aufrufen    }
  pAnfang:= NIL;                            { und Zeiger auf erstes Element    }
                                            { mit NIL initialisieren           }
  pAktuell:= NIL;                           { und Zeiger auf aktuelles Element }
                                            { mit NIL initialisieren           }
  pIsBefore:= true;
  pIsBehind:= false;
end;

destructor TLinList.destroy;                { Destruktor                       }
begin
  toFirst;
  while not isEmpty do delete;              { alle Listenelemente lschen      }
  inherited destroy;                        { und schlielich den geerbten     }
                                            { Destruktor aufrufen              }
end;

procedure TLinList.next;                    { einen Datensatz vor              }
begin
  if (not isBehind)
  then begin
         if isEmpty
            then begin
                   pIsBehind:= true;
                   pIsBefore:= false;
                 end
            else if isBefore
                 then begin
                        pAktuell:= pAnfang;
                        pIsBefore:= false;
                      end
                 else begin
                        pAktuell:= pAktuell.Next;
                        pIsBefore:= false;
                        if pAktuell=nil
                        then pIsBehind:= true;
                      end;
       end;
end;

procedure TLinList.previous;                { einen Datensatz zurck           }
var h: TElement;
begin
  if (not isBefore)
  then begin
         pIsBehind:= false;
         if isEmpty
         then pIsBefore:= true
         else if pAktuell = pAnfang
              then begin
                     pAktuell:= nil;
                     pIsBefore:= true;
                   end
              else begin
                     h:= pAnfang;
                     while h.Next<>pAktuell do h:= h.Next;
                     pAktuell:= h;
                   end;
       end;
end;

procedure TLinList.toFirst;                 { ersten Datensatz anwhlen        }
begin
  if isEmpty
  then begin
         pIsBehind:= true;
         pIsBefore:= false;
       end
  else begin
         pAktuell:= pAnfang;
         pIsBefore:= false;
         pIsBehind:= false;
       end;
end;

procedure TLinList.toLast;                  { letzten Datensatz anwhlen       }
begin
  if isEmpty
  then begin
         pIsBehind:= false;
         pIsBefore:= true;
       end
  else begin
         toFirst;
         while pAktuell.Next<>nil do next;
         pIsBefore:= false;
         pIsBehind:= false;
       end;
end;

procedure TLinList.insertBefore(pObject: TObject); { Neues element in Liste vor     }
var e: Telement;                                   { Aktuell einfuegen              }
begin
  if (not isBefore)
  then begin
         if isEmpty                                { Wenn Liste noch leer           }
         then begin
                e:= Telement.create(pObject);
                pAnfang:= e;                       { dann El. als erstes einreihen  }
              end
         else if pAktuell=pAnfang                  { Aktuell steht auf erstem Elem. }
              then begin
                     e:= Telement.create(pObject);
                     e.SetNext(pAnfang);
                     pAnfang:= e;
                   end
              else begin                           { Aktuell steht irgendwo sonst   }
                     previous;                     { erst ein Element zurck und    }
                     insertBehind(pObject);        { dann hinter Aktuell einfgen   }
                     next;
                     next;
              end;
       end;
end;

procedure TLinList.insertBehind(pObject: TObject); { Neues element in Liste nach    }
var e: TElement;                                   { Aktuell einfuegen              }
begin
  if (not isBehind)
  then begin
         if isEmpty                                { Wenn Liste noch leer           }
         then begin
                e:= Telement.create(pObject);
                pAnfang:= e;                       { dann El. als erstes einreihen  }
              end
         else if isBefore
              then begin
                     e:= TElement.create(pObject);
                     e.setNext(pAnfang);
                     pAnfang:= e;
                   end
              else begin                           { sonst Element hinter aktuellem }
                     e:= Telement.create(pObject); { Element einreiehen             }
                     e.setNext(pAktuell.next);
                     pAktuell.setNext(e);
              end;
       end;
end;

procedure TLinList.delete;                    { Element aus Liste lschen      }
var h: TElement;
begin
  if (not isBefore) and (not isBehind)
  then begin                                  { nur wenn Aktuell auf El. steht }
         if pAktuell=pAnfang                  { Wenn erstes Elem.gelscht wird,}
         then begin
                pAnfang:= pAnfang.Next;       { dann Anfang verschieben        }
                pAktuell.free;                { Aktuelles Element lschen      }
                pAktuell:= pAnfang;           { neues Aktuelles Element        }
                if pAktuell=nil
                then pIsBehind:= true;
              end
         else begin
                h:= pAktuell;                 { aktuelles Element merken       }
                previous;                     { zurueck gehen, um auszuketten  }
                pAktuell.setnext(h.next);     { bisheriges akt. Elem. ausketten}
                h.free;                       { bisheriges akt. Elem. lschen  }
                next;                         { dann das nachfolgende nehemen  }
              end;
       end
end;

procedure TLinList.update(pObject: TObject);  { Ersetzt den aktuellen Inhalt   }
begin
  if (not isEmpty) and (not isBefore) and (not isBehind)
  then begin
         pAktuell.setInhalt(pObject);
       end;
end;

function TLinList.getItem: TObject;           { Rckgabe: aktuelles Element    }
begin
  if isBefore or isBehind or isEmpty
  then result:= nil
  else result:= pAktuell.Inhalt;
end;

function TLinList.isEmpty: boolean;           { Liste leer?                    }
begin
  result:= (pAnfang=NIL);                     { Anfang = NIL => Liste leer     }
end;

function TLinList.isBehind: boolean;          { steht Aktuell hinter der Liste }
begin
  result:= pIsBehind;
end;

function TLinList.isBefore: boolean;          { steht Aktuell vor der Liste    }
begin
  result:= pIsBefore;
end;


end.
